﻿using System;
using Polly;
using Polly.Retry;

class Program
{
    static void Main(string[] args)
    {
        ErrorProneCode errorProneCode = new ErrorProneCode();
        RetryPolicy waitAndRetryPolicy = Policy.Handle<Exception>()
            .WaitAndRetry(5,
            sleepDurationProvider: (retryAttempt, ex, ctx) =>
            {
                if (ex is InsufficientMemoryException)
                {
                    return TimeSpan.FromSeconds(1);
                }
                else
                {
                    return TimeSpan.FromSeconds(2);
                }
            },
            onRetry: (exception, timeSpan, retryCount, ctx) =>
            {
                Console.WriteLine($"Got a response of {exception.Message}, retrying {retryCount}. Delaying for {timeSpan}");
            }
        );

        string result = waitAndRetryPolicy.Execute(errorProneCode.DoSomethingThatMightThrowException);
        Console.WriteLine(result);
    }
}
